<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Services\GolfManagerService;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;

class AvailabilityController extends Controller
{
    public function index()
    {
        $start = Carbon::now();

        if (request('start')) {
            $start = Carbon::parse(request('start'));

            session([
                'search_availability' => request('start'),
            ]);
        }

        $reservations = GolfManagerService::getReservations([
            'clientId' => request('client'),
            'start' => $start->copy()->subHours(12),
            'end' => $start->copy()->addHours(12),
        ])->map(function ($reservation) {
            return [
                'date' => Carbon::parse($reservation['start'])->setTimezone('Asia/Muscat'),
                'name' => Auth::getUser()->isGuest() ? 'Booked' : $reservation['clientName'],
            ];
        })->toArray();

        $slots = collect(GolfManagerService::getAvailability([
            'start' => $start,
        ]))->map(function ($slot) use ($reservations) {
            $slots = [];

            $slotDate = Carbon::parse($slot['date'])->setTimezone('Asia/Muscat');

            for ($i = 0; $i < $slot['slots']; $i++) {
                $slots[] = [
                    'date' => $slotDate,
                    'name' => 'Free',
                ];
            }

            foreach($reservations as $reservation) {
                if ($slotDate->eq($reservation['date'])) {
                    $slots[] = $reservation;
                }
            }

            $slot['slots'] = $slots;

            return $slot;
        });

        return [
            'slots' => $slots,
        ];
    }
}
