<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Jobs\SendFoodOrderEmail;
use App\Jobs\SendNewBookingEmail;
use App\Jobs\SendTableReservationEmail;
use App\Services\BookingSession;
use App\Services\GolfManagerService;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Redirect;

class PaymentController extends Controller
{
    public function index()
    {
        $booking = BookingSession::get();


        $userDetails = Auth::user();

        if (Auth::getUser()->isGuest()) {
            $primaryGuest = $booking->getPrimaryGuest();

            $userDetails = [
                'name' => $primaryGuest['first_name'] . ' ' . $primaryGuest['last_name'],
                'email' => $primaryGuest['email'],
            ];
        }

        $golfManagerSession = GolfManagerService::confirmReservation($booking);

        if (isset($reservation['error'])) {
            return Redirect::route('booking.complete.index', [
                'booking' => $booking->id,
            ])->withErrors($reservation['error']);
        }

        $booking = $booking->update([
            'golf_manager_session' => $golfManagerSession,
        ]);

        SendNewBookingEmail::dispatch($booking, $userDetails);

        if (count($booking->getThawaniProducts(['foodOnly' => true]))) {
            SendFoodOrderEmail::dispatch($booking, $userDetails);
        }

        if ($booking->tableReservation) {
            SendTableReservationEmail::dispatch($booking, $userDetails);
        }

        return Redirect::route('booking.success.index', [
            'booking' => $booking->id,
        ]);
    }
}
