<?php

namespace App\Http\Controllers\Application;
use App\Http\Controllers\Controller;
use App\Http\Requests\DeclarationRequest;
use App\Services\ThawaniService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Redirect;
use Inertia\Inertia;
use Ramsey\Uuid\Uuid;

class DeclarationController extends Controller
{
    public function index()
    {
        if (!session('application')) {
            return Redirect::route('signup.index');
        }

        return Inertia::render('Application/Declaration', [
            'application' => session('application'),
        ]);
    }

    public function store(DeclarationRequest $request)
    {
        $application = array_merge(session('application', []), $request->all());

        $application['id'] = Uuid::uuid4()->toString();

        $application['photo']->storeAs('/' . $application['id'], 'photo.' . $application['photo']->clientExtension());
        unset($application['photo']);

        $application['signature']->storeAs('/' . $application['id'], 'signature.' . $application['signature']->clientExtension());
        unset($application['signature']);

        if ($application['tenancy']) {
            $application['tenancy']->storeAs('/' . $application['id'], 'tenancy.' . $application['tenancy']->clientExtension());
            unset($application['tenancy']);
        }

        $application['name'] = $application['first_name'] . ' ' . $application['last_name'];
        $applicants = count($application['applicants']);

        if ($applicants == 0) {
            $applicants++;
        }

        for ($i = 1; $i <= $applicants; $i++) {
            $application['products'] = [
                [
                    'name' => 'Quaterly Membership ' . $application['membership_type'] . ' ' . $application['membership_category'],
                    'quantity' => 4,
                    'unit_amount' => floor(($application['cost'] * 1000) / 4),
                ]
            ];
        }

        session([
            'application' => $application,
        ]);

        $application['thawani'] = ThawaniService::startApplicationSession($application);

        if (isset($application['thawani']['error'])) {
            return back();
        }

        session([
            'application' => $application,
        ]);

        return Inertia::location(
            config('services.thawani.url') . '/pay/' . $application['thawani']['session_id'] . '?key=' . config('services.thawani.key')
        );
    }
}

