<?php

namespace App\Http\Controllers\Application;

use App\Http\Controllers\Controller;
use App\Http\Requests\ApplicationRequest;
use App\Services\NationalityService;
use Illuminate\Support\Facades\Redirect;
use Inertia\Inertia;
use Monarobase\CountryList\CountryListFacade;
use Symfony\Component\Yaml\Yaml;

class DetailsController extends Controller
{
    public function index()
    {
        return Inertia::render('Application/Index', [
            'applicants' => request('applicants'),
            'nationalities' => NationalityService::all(),
            'countries' => collect(CountryListFacade::getList('en'))->filter(fn($country) => $country != 'Oman')->map(function ($country, $key) {
                return ['id' => 'option-' . $key, 'label' => $country, 'value' => $country];
            })->values()->prepend(['id' => 'option-Oman', 'label' => 'Oman', 'value' => 'Oman']),
        ]);
    }

    public function store(ApplicationRequest $request)
    {
        $application = $request->all();

        $memberships = Yaml::parse(file_get_contents(resource_path('yaml/memberships.yaml')));

        $application['cost'] = $memberships[$application['membership_type']][$application['membership_category']] ?? 0;

        if ($application['cost'] == 0) {
            return back()->withErrors([
                'cost' => 'Could not find a cost for this membership type.',
            ]);
        }

        session([
            'application' => $application,
        ]);

        return Redirect::route('signup.declaration.index');
    }
}
