<?php

namespace App\Http\Controllers;

use App\Http\Requests\AuthRequest;
use App\Jobs\AddUserToThawani;
use App\Models\User;
use App\Services\GolfManagerService;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Session;
use Inertia\Inertia;

class AuthController extends Controller
{
    public function login(AuthRequest $request)
    {
        if (!$userData = GolfManagerService::login($request->get('email'), $request->get('password'))) {
            return back()->withErrors([
                'login' => 'Invalid credentials',
            ]);
        }

        if (!$user = User::whereEmail($request->get('email'))->first()) {
            $user = new User();

            $user->name = $userData['name'];
            $user->email = $request->get('email');
            $user->golf_manager_id = $userData['id'];
            $user->password = Crypt::encrypt('golf_manager_token');
        }

        $user->password = Crypt::encrypt($request->get('password'));
        $user->save();

        Auth::login($user);

        session([
            'user_meta' => $userData,
        ]);

        if (!$user->thawani_id) {
            AddUserToThawani::dispatch($user);
        }

        return Redirect::route('availability.index');
    }

    public function logout()
    {
        Auth::logout();

        Session::flush();


        Cache::flush();

        return Inertia::location('https://www.almoujgolf.com/');
    }

    public function guest()
    {
        Auth::loginUsingId(1);

        return Redirect::route('availability.index');
    }
}
