<?php

namespace App\Http\Controllers\Booking;

use App\Http\Controllers\Controller;
use App\Services\BookingSession;
use App\Services\GolfManagerService;
use App\Services\ThawaniService;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Redirect;
use Inertia\Inertia;

class CompleteController extends Controller
{
    public function index()
    {
        return Inertia::render('Booking/Complete');
    }

    public function store()
    {
        $booking = BookingSession::get();

        $golfManagerSession = GolfManagerService::makeReservation($booking);

        if (isset($golfManagerSession['error'])) {
            return back()->withErrors($golfManagerSession['error']);
        }

        $booking = $booking->update([
            'id' => $golfManagerSession['reservations'][0]['reference'] ?? $booking->id,
            'golf_manager_session' => $golfManagerSession,
        ]);

        if (count($booking->products) || Auth::getUser()->isGuest()) {
            $thawaniSession = ThawaniService::startBookingSession($booking);

            if (isset($thawaniSession['error'])) {
                return back()->withErrors([
                    'thawani' => json_encode($thawaniSession['error'])
                ]);
            }

            if ($thawaniSession['payment_status'] == 'paid') {
                return Redirect::route('availability.index');
            }

            $booking = $booking->update([
                'thawani_session_id' => $thawaniSession['session_id'],
            ]);

            if ($booking->thawani_session_id) {
                return Inertia::location(
                    config('services.thawani.url') . '/pay/' . $booking->thawani_session_id . '?key=' . config('services.thawani.key')
                );
            }

            return back()->withErrors([
                'thawani' => 'Error creating Thawani payment session.',
            ]);
        }

        return Redirect::route('api.payment.index');
    }
}
