<?php

namespace App\Http\Controllers\Booking;

use App\Http\Controllers\Controller;
use App\Http\Requests\SaveGuestRequest;
use App\Services\BookingSession;
use App\Services\GolfManagerService;
use App\Services\NationalityService;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Redirect;
use Inertia\Inertia;
use Monarobase\CountryList\CountryListFacade;

class GuestController extends Controller
{
    public function index()
    {
        $guest = null;
        if (Auth::getUser()->isGuest()) {
            $guest = GolfManagerService::searchUsers('Guest User')->first();
        }

        return Inertia::render('Booking/Guest', [
            'prices' => GolfManagerService::getPrices($guest ? $guest['idTags'] : null, true),
            'countries' => collect(CountryListFacade::getList('en'))->filter(fn($country) => $country != 'Oman')->map(function ($country, $key) {
                return ['id' => 'option-' . $key, 'label' => $country, 'value' => $country];
            })->values()->prepend(['id' => 'option-Oman', 'label' => 'Oman', 'value' => 'Oman']),
            'nationalities' => NationalityService::all(),
        ]);
    }

    public function store(SaveGuestRequest $request)
    {
        $booking = BookingSession::get();

        $booking->update([
            'guests' => $request->all(),
        ]);

        if ($booking->hasGuests()) {
            return Redirect::route('booking.hire.index', [
                'booking' => session('booking.id'),
            ]);
        }

        return Redirect::route('booking.food.index', [
            'booking' => session('booking.id'),
        ]);
    }
}
