<?php

namespace App\Http\Controllers;

use App\Http\Requests\PasswordResetRequest;
use App\Services\GolfManagerService;
use Illuminate\Support\Facades\Password;
use Illuminate\Http\Request;
use Inertia\Inertia;

class PasswordController extends Controller
{
    public function index()
    {
        return Inertia::render('Password/Index');
    }

    public function store(Request $request)
    {
        $request->validate(['email' => 'required|email']);

        Password::sendResetLink(
            request()->only('email')
        );

        session()->flash('message', 'Password reset email has been sent.');

        return back();
    }

    public function show($token)
    {
        return Inertia::render('Password/Resets', [
            'token' => $token,
            'email' => request('email'),
        ]);
    }

    public function update(PasswordResetRequest $request)
    {
        Password::reset(
            $request->all(),
            function ($user, $password) {
                GolfManagerService::resetPassword($user->golf_manager_id, $password);
            }
        );

        return Inertia::render('Password/Success');
    }
}
