<?php

namespace App\Http\Middleware;

use App\Services\BookingSession;
use App\Services\GolfManagerService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Inertia\Middleware;

class HandleInertiaRequests extends Middleware
{
    /**
     * The root template that's loaded on the first page visit.
     *
     * @see https://inertiajs.com/server-side-setup#root-template
     * @var string
     */
    protected $rootView = 'app';

    /**
     * Determines the current asset version.
     *
     * @see https://inertiajs.com/asset-versioning
     * @param  \Illuminate\Http\Request  $request
     * @return string|null
     */
    public function version(Request $request)
    {
        return parent::version($request);
    }

    /**
     * Defines the props that are shared by default.
     *
     * @see https://inertiajs.com/shared-data
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
    public function share(Request $request)
    {
        return array_merge(parent::share($request), [
            'user' => fn () => Auth::user(),
            'booking' => fn () => BookingSession::get(),
            'flash' => [
                'message' => fn () => $request->session()->get('message')
            ],
            'golf_manager_product_images' => config('services.golfmanager.product_images_url'),
            'prices' => function () {
                if (!Auth::user()) {
                    return null;
                }

                $bookingProducts = GolfManagerService::getBookingProducts();

                $clubProduct = $bookingProducts->first(function ($bookingProduct) {
                    return $bookingProduct['name'] === '18 Hole Rental clubs';
                });

                return [
                    'clubs' => $clubProduct ? ($clubProduct['price'] * 1000) : 15750,
                ];
            },
        ]);
    }
}
