<?php

namespace App\Services;

use App\Models\User;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Http;

class ThawaniService
{
    private static function get($url, $query = [])
    {
        return Http::withHeaders([
            'thawani-api-key' => config('services.thawani.secret'),
        ])->get(config('services.thawani.url') . '/api/v1' . $url, $query);
    }

    private static function post($url, $data = [])
    {
        return Http::withHeaders([
            'Content-Type' => 'application/json',
            'thawani-api-key' => config('services.thawani.secret'),
        ])->post(config('services.thawani.url') . '/api/v1' . $url, $data);
    }

    public static function getCustomers()
    {
        return self::get('/customers');
    }

    public static function getCustomer($id)
    {
        return self::get(`/customers/{$id}`);
    }

    public static function createCustomer(User $user): string | null
    {
        return self::post('/customers', [
            'client_customer_id' => $user->email,
        ])->json()['data']['id'] ?? null;
    }

    public static function startBookingSession(BookingSession $booking)
    {
        $user = Auth::user();

        if (!$user) {
            throw new \Exception('Cannot start Thawani session without an authenticated user.');
        }

        $session = self::get("/checkout/reference/{$booking->id}")->json();

        if (!array_key_exists('data', $session) || $session['code'] != 2000) {
            $primaryGuest = null;

            if (Auth::getUser()->isGuest()) {
                $primaryGuest = $booking->getPrimaryGuest();
            }

            $session = self::post('/checkout/session', [
                'client_reference_id' => $booking->id,
                'mode' => 'payment',
                'products' => $booking->getThawaniProducts(),
                'customer_id' => Auth::getUser()->isGuest() ? '' : $user->thawani_id,
                'success_url' => url("/api/payment/complete"),
                'cancel_url' => url("/booking/{$booking->id}/complete"),
                'metadata' => [
                    'name' => $primaryGuest ? ($primaryGuest['first_name'] . ' ' . $primaryGuest['last_name']) : $user->name,
                    'phone' => $primaryGuest ? $primaryGuest['phone'] : $user->phone ?? '',
                    'email' => $primaryGuest ? $primaryGuest['email'] : $user->email,
                ],
            ])->json();
        }

        return $session['data'];
    }

    public static function startApplicationSession($application)
    {
        $session = self::get("/checkout/reference/{$application['id']}")->json();

        if (!array_key_exists('data', $session) || $session['code'] != 2000) {
            $session = self::post('/checkout/session', [
                'client_reference_id' => $application['id'],
                'mode' => 'payment',
                'products' => $application['products'],
                'success_url' => url("/api/membership/complete"),
                'cancel_url' => url("/signup/declaration"),
                'metadata' => [
                    'name' => $application['name'],
                    'phone' => $application['phone'] ?? '',
                    'email' => $application['email'],
                ],
            ])->json();
        }

        return $session['data'];
    }
}
