import React, { useState } from 'react';
import { usePage, InertiaLink } from "@inertiajs/inertia-react";
import { bookingHasGuests, formatAsCurrency } from "../funcions";
import { Inertia } from '@inertiajs/inertia';
import NavStepItem from './NavStepItem';
import NavSteps from './NavSteps';

export default function Nav() {
  const { user, booking } = usePage().props;
  const [visibleSteps, setVisibleSteps] = useState(0);

  const steps = [
    {
      name: 'Hire',
      href: `/booking/${booking?.id}/hire`,
      content: <>Equipment<br/>Rental</>,
      active: true,
    },
    {
      name: 'Food',
      href: `/booking/${booking?.id}/food`,
      content: <>Food &<br/>Beverage</>,
      complete: true,
    },
    {
      name: 'Guests',
      href: `/booking/${booking?.id}/guests`,
      content: <>Guest<br/>Details</>,
    },
    {
      name: 'Complete',
      href: `/booking/${booking?.id}/complete`,
      content: <>Complete<br/>Booking</>,
    }
  ];

  function logout() {
    Inertia.post('/logout');
  }

  return (
    <header className="flex flex-col">
      <div className="flex justify-center">
        <div className="bg-white flex flex-col w-full max-w-[1344px] px-4">
          <section className="flex justify-between items-center w-full py-4 md:py-8">
            <InertiaLink href="/"><img className="md:h-[100px] w-15.5 md:w-auto" src="/img/logo.svg" /></InertiaLink>
              {booking && <>
                <NavSteps>
                  {steps.map((step, key) => {
                    if (!step.visible) {
                      return;
                    }

                    setVisibleSteps(visibleSteps + 1)

                    return <NavStepItem href={step.href} step={visibleSteps} active={step.active} complete={step.complete}>{step.content}</NavStepItem>
                  })}
                </NavSteps>
              </>}
            <button className="w-[60px] h-12 md:h-auto md:w-[100px] md:py-4 btn btn--outline" onClick={logout}><span>Exit</span></button>
          </section>
        </div>
      </div>

      {(user && user?.name != 'Guest') && <div className="flex justify-center bg-blue-900">
        <div className="flex flex-col w-full max-w-[1344px] px-[15px]">
          <section className="w-full flex justify-between items-center py-2 text-white">
            <div className="flex space-x-2"><p>{ user?.name }</p><p>{ user?.type || 'Standard User' }</p></div>
            <div className="flex space-x-2"><p>Account Balance</p><p>{ formatAsCurrency(user?.balance || 0) } OMR</p></div>
          </section>
        </div>
      </div>}
    </header>
  )
}
