import { DateTime } from 'luxon';
import React, {useEffect, useState} from 'react';
import { getBasketTotal } from '../funcions';
import { Collapse } from 'react-collapse';
import { map } from 'lodash';

export default function Basket({ booking, onNext, error, disabled, prices }) {
  const [isCartOpened, setIsCartOpened] = useState(true);
  const [isMobile, setIsMobile] = useState({
    matches: window.innerWidth >= 1024 ? true : false,
  });

  const {products} = booking;

  function toggleCart () {
    return setIsCartOpened(!isCartOpened);
  }

  useEffect( async () => {
    let mediaQuery = window.matchMedia("(min-width: 1024px)");
    mediaQuery.addEventListener('change', setIsMobile);

    setIsCartOpened(isMobile.matches);
    // this is the cleanup function to remove the listener
    return () => mediaQuery.removeEventListener('change', setIsMobile);
  }, [isMobile]);

  return (
    <div className="basket flex flex-col lg:absolute right-[15px] top-16 w-full lg:w-[416px] bg-blue-900 text-white px-4 pt-3 lg:p-8 lg:pt-6.5">
      <h2 className="uppercase mb-2 md:mb-8 tracking-widest text-golden text-[13px]">Booking Summary {booking?.golf_manager_id && ('#' + booking?.golf_manager_id)}</h2>

      <Collapse isOpened={ isCartOpened }>
        <div className='text-xs space-y-3 pb-4 md:pb-0 flex flex-col overflow-y-auto hidden-scroll'>
        <div className="flex justify-between"><span className="w-36 pr-2">Date</span> <span className="text-sm leading-none text-right">{DateTime.fromISO(booking.start).toLocaleString(DateTime.DATE_HUGE)}</span></div>
        <div className="flex justify-between"><span className="w-36 pr-2">Tee Time</span> <span className="text-sm leading-none text-right">{DateTime.fromISO(booking.start).toLocaleString(DateTime.TIME_24_SIMPLE)}</span></div>

        <div className="flex justify-between">
          <span className="w-36 pr-2">Guests</span>
          {(!!booking?.guests?.adults?.length || !!booking?.guests?.juniors?.length) ? <p className="text-sm leading-none text-right">
            {!!booking?.guests?.adults ? `${booking.guests.adults.length} Adult${(booking.guests.adults.length > 1) ? 's' : ''}` : ''}
            {(!!booking?.guests?.adults?.length && !!booking?.guests?.juniors?.length) && ', '}
            {!!booking?.guests?.juniors?.length ? `${booking.guests.juniors.length} Junior${(booking.guests.juniors.length > 1) ? 's' : ''}` : ''}
          </p> : <span className="text-gray-500">No Guests</span>}
        </div>

        {!!booking?.tableBooking && <div className="flex justify-between">
          <span className="w-36 pr-2">Table Booking</span>
          <span className="text-sm tracking-widest leading-none text-right">{DateTime.fromISO(booking.tableBooking).toFormat('HH:mm')}</span>
        </div>}

        {!!products?.clubs && <div className="flex justify-between ">
          <span className="w-36 pr-2">Equipment Rental</span>
          <p className="space-y-1 text-right text-sm">
            {`${products.clubs} Set${(products.clubs > 1) ? 's' : ''} of Clubs`}
          </p>
        </div>}

        {!!Object.keys(products?.food || {}).length && <div className="flex justify-between">
          <span className="w-36 pr-2">Food</span>
          <ul className="space-y-1 text-right text-sm">
            {map(products?.food, (food, index) => {
              return <li key={index} className="flex flex-col">
                <h3 className="text-sm tracking-widest leading-none">{food.name}</h3>
                <p className="text-xs opacity-60 pt-1.5 tracking-widest">Qty: {food.quantity}</p>
              </li>
            })}
          </ul>
        </div>}

        {!!Object.keys(products?.beverages || {}).length && <div className="flex justify-between">
          <span className="w-36 pr-2">Beverages</span>
          <ul className="space-y-1 text-right text-sm">
            {map(products?.beverages, (beverage, index) => {
              return <li key={index} className="flex flex-col">
                <h3 className="text-sm tracking-widest leading-none">{beverage.name}</h3>
                <p className="text-xs opacity-60 pt-1.5 tracking-widest">Qty: {beverage.quantity}</p>
              </li>
            })}
          </ul>
        </div>}
      </div>
      </Collapse>
      <div className="basket__total flex justify-between pb-1 lg:pb-0 mt-0 md:mt-20 lg:mt-[133px] text-xs leading-30">
        <button className="flex items-center tracking-wide mr-auto lg:hidden" onClick={ toggleCart }>
          Show details
          <svg className={`${isCartOpened ? '-scale-y-100' : ''} text-green-500 ml-3 transform lg:hidden`}  width="10" height="6" viewBox="0 0 10 6" fill="none" xmlns="http://www.w3.org/2000/svg">
            <path d="M1 1L5 5L9 1" stroke="currentColor" strokeLinecap="round" strokeLinejoin="round"/>
          </svg>
        </button>
        <span className="lg:w-25 pr-2.5 lg:pr-2">Total cost</span>
        <span className="text-base flex items-baseline">
          <span className="truncate max-w-[100px]">{getBasketTotal(booking, prices)}</span> <span className="text-xs ml-1">OMR</span>
        </span>
      </div>

      <div className="-mx-4 lg:mx-0">
        {onNext && <button type='button' disabled={disabled} className={`flex justify-between px-4 lg:px-8 py-4 w-full mt-1.5 md:mt-6.5 btn ${disabled ? 'bg-gray-400' : 'btn--green'}`} onClick={onNext}>
          Continue
          <svg width="16" height="12" viewBox="0 0 16 12" fill="none" xmlns="http://www.w3.org/2000/svg">
            <path d="M15 6L1 6M15 6L9.4 11M15 6L9.4 1" stroke="currentColor" strokeLinecap="round" strokeLinejoin="round"/>
          </svg>
        </button>}
      </div>

      {error && <p className="text-red-error my-2">{error}</p>}
    </div>
  )
}
