
import React, { useState } from "react";
import Select, { components } from "react-select";
import { Scrollbars } from "react-custom-scrollbars-2";
import AsyncSelect from 'react-select/async';

const MenuList = ( props ) => {
  return (
    <div className="h-[200px] z-100">
      <Scrollbars
        renderTrackVertical={props => <div {...props} className="track-vertical bg-gray-200 right-2 top-2 bottom-2 max-w-[1px]"/>}
        renderThumbVertical={props => <div {...props} className="thumb-vertical bg-gray-400"/>}
      >
        { props.children }
      </Scrollbars>
    </div>
  );
};

const DropdownIndicator = ( props ) => {
  return (
    <components.DropdownIndicator {...props}>
      <svg className="text-green-500" width="14" height="8" viewBox="0 0 14 8" fill="none" xmlns="http://www.w3.org/2000/svg">
        <path d="M1 1L7 7L13 1" stroke="currentColor" strokeLinecap="round" strokeLinejoin="round"/>
      </svg>
    </components.DropdownIndicator>
  );
};

const IndicatorSeparator = ( props ) => {
  return null;
}

export const CustomSelect = (props) => {
  return (
    <Select
      styles={''}
      components={{ MenuList, DropdownIndicator, IndicatorSeparator }}
      placeholder={props.placeholder || '-'}
      captureMenuScroll={false}
      className={'react-select ' + (props.className || '')}
      classNamePrefix={'react-select'}
      value={props?.value}
      options={props.options}
      onChange={props.onChange || (() => {})}
      defaultValue={props.defaultValue || null}
      // menuIsOpen={true}
    />
  );
};

export const SearchSelect = (props) => {
  const [inputValue, setInputValue] = useState('');

  const handleInputChange = (newValue) => {
    setInputValue(newValue.replace(/\W/g, ''));
  };

  return (
    <AsyncSelect
      styles={''}
      components={{ MenuList, DropdownIndicator, IndicatorSeparator }}
      placeholder={props.placeholder || '-'}
      captureMenuScroll={false}
      className={'react-select ' + (props.className || '')}
      classNamePrefix={'react-select'}
      value={props?.value}
      options={props.options}
      onChange={props.onChange || (() => {})}
      defaultValue={props.defaultValue || null}
      // menuIsOpen={true}
      onInputChange={handleInputChange}
      loadOptions={props.loadOptions}
      defaultOptions={props.defaultOptions}
    />
  );
};
