import React, { useEffect, useState } from 'react';

export default function Input({
  inputWrapperStyle,
  inputLabelStyle,
  inputStyle,
  required,
  label,
  name,
  value,
  onChange,
  type,
  message,
  error,
  validate,
}) {
  const [inputState, setInputState] = useState('default')
  useEffect(() => {
    if (validate ? validate(value) : value != '') {
      setInputState('success');
    }
  }, [value])

  return (
    <div className={`flex flex-col relative ${inputWrapperStyle}`}>
      {/* do not remove placeholder */}
      <input
        type={type || 'text'}
        id={name} value={value}
        onChange={onChange}
        name={name}
        className={`border-r-0 border-t-0 border-l-0 focus:outline-none focus:ring-0 active:outline-none border-b py-1 px-2 h-9 text-[13px] sm:text-[15px] outline-none tracking-wider ${inputStyle} ${inputState === 'error' && ' border-red-error focus:border-red-error'} ${inputState === 'default' && ' focus:border-gray-500 border-gray-500 border-opacity-40'} ${inputState === 'success' && ' focus:border-green-500 border-green-500'}`} placeholder=' ' />
      <label htmlFor={name} className={`pointer-events-none uppercase text-[10px] sm:text-[13px] tracking-widest anim-label ${inputLabelStyle}`}>{ label || '' } {required && <span className="text-golden">*</span>}</label>
      {inputState === 'success' &&
        <div className="absolute right-[11px] top-[13px] w-1.5 text-green-500 field-state">
          <svg width="12" height="9" viewBox="0 0 12 9" fill="none" xmlns="http://www.w3.org/2000/svg">
            <path d="M11.0001 1L4.00004 8.00002L1 5" stroke="currentColor" strokeLinecap="round" strokeLinejoin="round"/>
          </svg>
        </div>
      }
      {(inputState === 'error' || error) &&
        <div className="absolute right-[11px] top-[13px] w-1.5 text-red-error">
          <svg width="2" height="12" viewBox="0 0 2 12" fill="none" xmlns="http://www.w3.org/2000/svg">
            <path d="M1.5 1C1.5 0.723858 1.27614 0.5 1 0.5C0.723858 0.5 0.5 0.723858 0.5 1H1.5ZM0.5 8C0.5 8.27614 0.723858 8.5 1 8.5C1.27614 8.5 1.5 8.27614 1.5 8H0.5ZM1.5 10.99C1.5 10.7139 1.27614 10.49 1 10.49C0.723858 10.49 0.5 10.7139 0.5 10.99H1.5ZM0.5 11C0.5 11.2761 0.723858 11.5 1 11.5C1.27614 11.5 1.5 11.2761 1.5 11H0.5ZM0.5 1V8H1.5V1H0.5ZM0.5 10.99V11H1.5V10.99H0.5Z" fill="currentColor"/>
          </svg>
        </div>
      }
      {error &&
        <span className={`absolute top-full left-0 text-xs tracking-wider pt-1 text-red-error`}>{error}</span>
      }
    </div>
  )
}
