import { each, merge } from "lodash";

export function formatAsCurrency(x) {
  return x.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
}

export function getBasketTotal(booking, prices = {}) {
  let {products} = booking;
  let total = 0;

  prices = merge({}, {
    clubs: 15000,
    adults: 45000,
    juniors: 35000,
  }, prices)

  each(products?.food || {}, (food, index) => {
    total += food.price * food.quantity;
  })

  each(products?.beverages || {}, (beverage, index) => {
    total += beverage.price * beverage.quantity;
  })

  if (products?.clubs) {
    total += (products?.clubs * (prices.clubs / 1000))
  }

  if (booking?.guests?.adults) {
    booking?.guests?.adults.forEach(adult => {
      if (adult.price != 'loading') {
        total += (adult.price ?? prices.adults) / 1000;
      }
    });
  }

  if (booking?.guests?.juniors) {
    booking?.guests?.juniors.forEach(junior => {
      if (junior.price != 'loading') {
        total += (junior.price ?? prices.juniors) / 1000;
      }
    });
  }

  return total.toFixed(3)
}

export function bookingHasGuests(booking) {
  return booking?.guests?.adults.length > 0 || booking?.guests?.juniors.length > 0;
}

export function validateEmail(email) {
  return String(email)
    .toLowerCase()
    .match(
      /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/
    );
}

export function dataURLtoFile(dataurl, filename) {
    var arr = dataurl.split(','),
        mime = arr[0].match(/:(.*?);/)[1],
        bstr = atob(arr[1]),
        n = bstr.length,
        u8arr = new Uint8Array(n);

    while(n--){
        u8arr[n] = bstr.charCodeAt(n);
    }

    return new File([u8arr], filename, {type:mime});
}
