import React from 'react';
import Bravo from '../../components/typography/Bravo';
import Main from '../../layouts/Main';
import { useForm } from '@inertiajs/inertia-react';
import { dataURLtoFile, formatAsCurrency } from '../../funcions';
import ReactSignatureCanvas from 'react-signature-canvas';

const Decalaration = ({ application }) => {
  let signature;
  let imageInput;

  const { data, setData, post, processing, errors } = useForm({
    photo: null,
    tenancy: null,
    signature: null,
    privacy_policy: false,
    cost: application.cost ?? 0,
  });

  async function submitDeclaration(e) {
    e.preventDefault();

    post('/signup/declaration');
  }

  return (
    <div className="flex justify-center w-full bg-gray-100">
      <div className="pt-[65px] max-w-[1344px] mx-auto self-center w-full px-[15px]">

        <Bravo className={'md:text-center'}>Submit your declaration</Bravo>

        <form onSubmit={submitDeclaration} className="bg-white px-4 py-8 md:p-8 max-w-[1088px] md:w-full space-y-[26px] mt-16 -mx-[15px] md:mx-auto lg:pl-[114px] lg:pr-[114px] lg:pt-16">
          <p>I, the undersigned, in submitting this application, agree to abide by the decision of the management, which shall be final.</p>

          <p>I further agree that should membership be approved, the individuals will be bound by the rules and regulations of Al Mouj Golf and will discharge all unpaid annual subscriptions and any other charges and debts incurred by the named and any subsequent name members.</p>

          <p>Membership is granted at the sole discretion of the management. Annual membership fees must be paid in full. It is important that all parts of this application form are completed.</p>

          <div className="grid grid-flow-row md:grid-cols-12 auto-rows-max gap-y-8 md:space-x-4">
            <div className="flex flex-col col-span-12 md:col-span-6 space-y-4">
              <div className="flex flex-col col-span-6 space-y-4">
                <p className='text-sm font-semibold'>Please upload a copy of your National ID Card/Passport <span className="text-golden">*</span></p>
                {errors.photo && <p className='text-red-error'>{errors.photo}</p>}
                {data.photo && <p>{data.photo.name}</p>}
                <label htmlFor="passportPhoto" className="btn btn--outline w-[240px]"><span>{!data.photo ? 'Upload' : 'Change'}</span></label>
                <input id="passportPhoto" type="file" className="hidden" onChange={e => setData('photo', e.target.files[0])} />
              </div>

              {application?.homeowner == 'false' && <div className="flex flex-col col-span-6 space-y-4">
                <p className='text-sm font-semibold'>Please upload a copy of your Tenancy Agreement <span className="text-golden">*</span></p>
                {errors.tenancy && <p className='text-red-error'>{errors.tenancy}</p>}
                {data.tenancy && <p>{data.tenancy.name}</p>}
                <label htmlFor="tenancyAgreement" className="btn btn--outline w-[240px]"><span>{!data.tenancy ? 'Upload' : 'Change'}</span></label>
                <input id="tenancyAgreement" type="file" className="hidden" onChange={e => setData('tenancy', e.target.files[0])} />
              </div>}
            </div>

            <input type='file' ref={ref => imageInput = ref} className='hidden' onChange={e => setData('signature', e.target.files[0])}/>

            <div className="flex flex-col col-span-12 md:col-span-6">
              <label className="flex flex-col uppercase">
                <span className="flex items-center pb-4 space-x-4">
                  <span>Applicant Signature</span>
                  <button className="btn btn--outline text-center w-[100px] flex justify-center py-1" type="button" onClick={() => signature.clear()}>
                    <span>Clear</span>
                  </button>
                </span>
              </label>
              {errors.signature && <p className='text-red-error mb-4'>{ errors.signature }</p>}
              <div className="flex-1">
                <ReactSignatureCanvas onEnd={() => {
                  setData('signature', dataURLtoFile(signature.getTrimmedCanvas().toDataURL('image/png'), 'signature.png'));
                }} ref={(ref) => { signature = ref }} canvasProps={{width: 500, height: 270, className: 'sigCanvas border flex-1 w-full max-w-[500px]'}} />
              </div>
            </div>
          </div>

          <div className="space-y-4">
            <div>
              <h3 className="tracking-widest text-[13px] text-gray-400 uppercase font-bold mb-[2px]">Applicants</h3>
              <p>{(application.applicants && application.applicants.length) + 1 || 1}</p>
            </div>
            <div>
              <h3 className="tracking-widest text-[13px] text-gray-400 uppercase font-bold mb-[2px]">Cost per Applicant</h3>
              <p>{formatAsCurrency(application.cost ?? 500)} <small>OMR</small></p>
            </div>
            <div>
              <h3 className="tracking-widest text-[13px] text-gray-400 uppercase font-bold mb-[2px]">Total Cost</h3>
              <p>{formatAsCurrency(application.cost * ((application.applicants && application.applicants.length) + 1 || 1) ?? 500)} <small>OMR</small></p>
            </div>
          </div>

          <div className="grid grid-cols-1 lg:grid-cols-12">
            <div className="flex col-span-12 lg:col-span-6 items-center space-x-2 mb-8 lg:mb-0">
              <div className="custom-checkbox">
                <input id="privacy_policy" checked={data.privacy_policy} onChange={e => {setData('privacy_policy', !data.privacy_policy)}} className="border-b p-2" type="checkbox"/>
                <span className="fake-checkbox" onClick={() => {
                  setData('privacy_policy', !data.privacy_policy)
                }}>
                  <svg width="12" height="9" viewBox="0 0 12 9" fill="none" xmlns="http://www.w3.org/2000/svg">
                    <path d="M11.0001 1L4.00004 8.00002L1 5" stroke="currentColor" strokeLinecap="round" strokeLinejoin="round"/>
                  </svg>
                </span>
                <label className="text-xs text-gray-500" htmlFor="privacy_policy">I agree to the <a href="https://www.almoujgolf.com/terms/" className="underline hover:no-underline" target="_blank">terms and conditions</a> and <a href="https://www.almoujgolf.com/privacy-policy/" target="_blank" className="underline hover:no-underline">privacy policy</a></label>
              </div>
            </div>

            <div className="flex col-span-6 space-y-4 md:justify-end">
              <button disabled={!data.privacy_policy} className={`${!data.privacy_policy ? 'bg-gray-200 cursor-not-allowed' : 'bg-green-500'} text-white px-8 py-4 w-[352px] flex justify-between items-center`}>
                Complete payment
                <svg xmlns="http://www.w3.org/2000/svg" className="h-6 w-6" fill="none" viewBox="0 0 24 24" stroke="currentColor">
                  <path strokeLinecap="round" strokeLinejoin="round" strokeWidth="2" d="M14 5l7 7m0 0l-7 7m7-7H3" />
                </svg>
              </button>
            </div>
          </div>
        </form>
      </div>
    </div>
  )
}

Decalaration.layout = page => <Main width='full' children={page} />

export default Decalaration
