import React from 'react';
import Main from '../../layouts/Main';
import Bravo from '../../components/typography/Bravo';
import { useForm } from '@inertiajs/inertia-react';
import Input from '../../components/input';
import Checkbox from '../../components/Checkbox';
import { useEffect } from 'react';
import { CustomSelect } from '../../components/customSelect';
import ReactTooltip from 'react-tooltip';
import 'react-phone-number-input/style.css';
import { validateEmail } from '../../funcions';
import PhoneInput from 'react-phone-number-input';
import PhoneTextInput from '../../components/PhoneTextInput';

const Index = ({ applicants, nationalities, countries }) => {
  const { data, setData, post, processing, errors } = useForm({
    membership_type: null,
    membership_category: null,
    email: '',
    first_name: '',
    last_name: '',
    company_name: '',
    job_title: '',
    address_line_1: '',
    address_line_2: '',
    city: '',
    school: '',
    zip_code: '',
    phone: '',
    gsm: '',
    passport: '',
    nationality: '',
    homeowner: false,
    handicap: '',
    dob: '',
    gender: '',
    applicants: [],
  });

  useEffect(() => {
    setData('applicants', []);

    if (applicants > 0) {
      addApplicant();
    }
  }, []);

  function submitApplication(e) {
    e.preventDefault();

    post('/signup');
  }

  useEffect(() => {
    if (data.membership_category != 'Family') {
      setData('applicants', []);
    }
  }, [data.membership_category])

  const handicapOptions = () => {
    let counts = [];

    for (let i = 0; i < 55; i++) {
      counts.push({
        id: `option-${i}`,
        value: i,
        label: i,
      });
    }

    return counts;
  };

  function addApplicant() {
    setData('applicants', [
      ...data.applicants,
      {
        first_name: '',
        last_name: '',
        email: '',
        nationality: '',
        gsm: '',
        gender: '',
        dob: '',
        phone: '+968',
        passport: '',
        handicap: '',
        homeowner: false,
      },
    ]);
  }

  function removeApplicant(i) {
    let applicants = data.applicants.filter((item, index) => {
      return index != i;
    });

    setData('applicants', applicants);
  }

  function updateApplicant(index, field, value) {
    let applicants = data.applicants;

    applicants[index][field] = value;

    setData('applicants', applicants);
  }

  function isJunior() {
    return data.membership_category === 'Student' || data.membership_category === 'Junior';
  }

  function isFamily() {
    return data.membership_category === 'Family';
  }

  return (
    <div className="flex justify-center w-full bg-gray-100">
      <div className="pt-[65px] max-w-[1344px] self-center w-full px-[15px]">
        <Bravo className="text-center">Apply to become a member of<br/> Al Mouj Golf course</Bravo>

        <form onSubmit={submitApplication} className="py-10 px-4 lg:px-[112px] lg:pt-16 bg-white space-y-6 my-16 -mx-[15px] md:mx-auto max-w-[1088px] md:w-full">
          <h2 className="text-lg font-semibold text-blue-900">Which membership options do you wish to apply for</h2>

          <div className="flex flex-col col-span-12 md:col-span-6 pb-[28px]">
            <label className="uppercase text-gray-500 font-bold text-[10px] sm:text-[13px] tracking-widest">Membership Type <span className="text-golden">*</span></label>

            <div className="flex flex-col md:flex-row md:space-x-12 space-y-8 md:space-y-0 mt-1.5">
              <Checkbox
                id="full_week"
                name="membership_type"
                label="Full Week Annual Membership"
                className="col-span-12 md:col-span-4 flex items-center space-x-2 text-[15px]"
                onChange={e => {
                  setData('membership_type', 'Full Week')
                }}
              />

              <Checkbox
                id="mid_week"
                name="membership_type"
                label="Mid Week Annual Membership (Sun - Thu)"
                className="col-span-12 md:col-span-4 flex items-center space-x-2 text-[15px]"
                onChange={e => {
                  setData('membership_type', 'Mid Week')
                }}
              />
            </div>
          </div>

          <div className="flex flex-col col-span-12 md:col-span-6 pb-[38px]">
            <label className="uppercase text-[10px] sm:text-[13px] tracking-widest">Membership Category <span className="text-golden">*</span></label>

            <div className="grid grid-cols-12 gap-y-6 mt-1.5">
              <Checkbox
                id="individual"
                name="membership_category"
                label="Individual"
                className="col-span-6 md:col-span-4 flex items-center space-x-2"
                onChange={e => {
                  setData('membership_category', 'Individual')
                }}
              />

              <Checkbox
                id="family"
                name="membership_category"
                label="Family"
                className="col-span-6 md:col-span-4 flex items-center space-x-2"
                onChange={e => {
                  setData('membership_category', 'Family')
                }}
              />

              <Checkbox
                id="junior"
                name="membership_category"
                label="Junior"
                className="col-span-6 md:col-span-4 flex items-center space-x-2"
                onChange={e => {
                  setData('membership_category', 'Junior')
                }}
              />

              <Checkbox
                id="student"
                name="membership_category"
                label="Student"
                className="col-span-6 md:col-span-4 flex items-center space-x-2"
                onChange={e => {
                  setData('membership_category', 'Student')
                }}
              />

              <Checkbox
                disabled={data.membership_type !== 'Full Week'}
                id="corporate"
                name="membership_category"
                label="Corporate"
                className="col-span-6 md:col-span-4 flex items-center space-x-2"
                onChange={e => {
                  setData('membership_category', 'Corporate')
                }}
              />

              <Checkbox
                disabled={data.membership_type !== 'Full Week'}
                id="corporate_plus"
                name="membership_category"
                label="Corporate Plus"
                className="col-span-6 md:col-span-4 flex items-center space-x-2"
                onChange={e => {
                  setData('membership_category', 'Corporate Plus')
                }}
              />
            </div>
          </div>

          <h3 className="text-lg pb-[25px] text-blue-900">Your details</h3>
          <div className="grid grid-cols-12 md:gap-x-8 gap-y-12 md:gap-y-16 pb-[40px]">
            <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.first_name} onChange={e => setData('first_name', e.target.value)} name="first_name" label="First Name" required={true} error={errors.first_name}/>
            <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.last_name} onChange={e => setData('last_name', e.target.value)} name="last_name" label="Last Name" required={true} error={errors.last_name}/>

            {!isJunior() && <Input
              inputWrapperStyle="col-span-12 md:col-span-6"
              value={data.company_name}
              onChange={e => setData('company_name', e.target.value)}
              name="company_name"
              label="Company Name"
              required={true}
            />}

            {isJunior() && <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.school} onChange={e => setData('school', e.target.value)} name="school" label="School" required={true} />}
            {!isJunior() && <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.job_title} onChange={e => setData('job_title', e.target.value)} name="job_title" label="Job Title" />}

            <Input
              inputWrapperStyle="col-span-12 md:col-span-6"
              value={data.email}
              onChange={e => setData('email', e.target.value)}
              name="email"
              label="Email"
              required={true}
              error={errors.email}
              validate={() => validateEmail(data.email)}
            />

            <div className="col-span-12 md:col-span-6 flex items-center">
              <PhoneInput
                required={true}
                label={'Phone'}
                international
                countryCallingCodeEditable={false}
                placeholder="Enter phone number"
                value={data.phone}
                onChange={e => setData(e)}
                defaultCountry={'OM'}
                style={{width: '100%', border: 'none'}}
                inputComponent={PhoneTextInput}
                error={errors.phone}
              />
            </div>

            <CustomSelect
              className="col-span-12 md:col-span-6 tracking-wider"
              options={nationalities}
              defaultValue={{ label: 'Nationality', value: null }}
              onChange={option => {
                setData('nationality', option.value)
              }}
            />

            <CustomSelect
              className="col-span-12 md:col-span-6"
              options={countries}
              defaultValue={{ label: 'Country of Residence', value: null }}
              onChange={option => {
                setData('country', option.value)
              }}
            />
          </div>

          <h3 className="text-[15px] text-blue-900">{!isJunior() ? 'Buisness address' : 'Address'}</h3>
          <div className="grid grid-cols-12 md:gap-x-8 gap-y-12 md:gap-y-16">
            <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.address_line_1} onChange={e => setData('address_line_1', e.target.value)} name="address_line_1" label="Address Line 1" required={true} error={errors.address_line_1}/>
            <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.address_line_2} onChange={e => setData('address_line_2', e.target.value)} name="address_line_2" label="Address Line 2" />
            <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.city} onChange={e => setData('city', e.target.value)} name="ctiy" label="City" />

            <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.zip_code} onChange={e => setData('zip_code', e.target.value)} name="zip_code" label="Zip Code" />

            <div className="flex flex-col col-span-12 md:col-span-6 space-y-2">
              <label className="uppercase text-[10px] sm:text-[13px] tracking-widest">Gender <span className="text-golden">*</span></label>

              <div className="flex items-center space-x-16">
                <Checkbox
                  id="male"
                  name="gender"
                  label="Male"
                  className="col-span-4 flex items-center space-x-2"
                  onChange={e => {
                    setData('gender', 'Male')
                  }}
                />

                <Checkbox
                  id="female"
                  name="gender"
                  label="Female"
                  className="col-span-4 flex items-center space-x-2"
                  onChange={e => {
                    setData('gender', 'Female')
                  }}
                />
              </div>

              {errors.gender &&
                <span className={`text-xs tracking-wider pt-1 text-red-error`}>{errors.gender}</span>
              }
            </div>

            <Input inputWrapperStyle="col-span-12 md:col-span-4 lg:col-span-3 md:mt-[26px]" value={data.dob} onChange={e => setData('dob', e.target.value)} name="dob" label="Date of birth" type="date" required={true} error={errors.dob}/>

            <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.passport} onChange={e => setData('passport', e.target.value)} name="passport" label="Resident ID or Passport Number" required={true} error={errors.passport}/>

            <div className='col-span-12 md:col-span-4 lg:col-span-3 flex items-center'>
              <CustomSelect
                className={"select-small flex-1"}
                options={handicapOptions()}
                placeholder={'Handicap'}
                onChange={(option) => {
                  setData('handicap', option.value)
                }}
              />
              <p data-tip="A golf handicap is a numerical measure of a golfer's potential that is used to enable players of varying abilities to compete against one another. Better players are those with the lowest handicaps.">
              <svg xmlns="http://www.w3.org/2000/svg" className="h-6 w-6 ml-4 text-green-500" fill="none" viewBox="0 0 24 24" stroke="currentColor">
                <path strokeLinecap="round" strokeLinejoin="round" strokeWidth="2" d="M13 16h-1v-4h-1m1-4h.01M21 12a9 9 0 11-18 0 9 9 0 0118 0z" />
              </svg>
              </p>
              <ReactTooltip className='max-w-[400px]'/>
            </div>

            <div className="flex flex-col col-span-12 md:col-span-6">
              <div className="space-x-2">
                <div className="custom-checkbox">
                  <input id="homeowner" checked={data.homeowner} onChange={e => setData('homeowner', e.target.checked)} className="border-b p-2" type="checkbox"/>
                  <span className="fake-checkbox" onClick={() => {
                    setData('homeowner', !data.homeowner)
                  }}>
                    <svg width="12" height="9" viewBox="0 0 12 9" fill="none" xmlns="http://www.w3.org/2000/svg">
                      <path d="M11.0001 1L4.00004 8.00002L1 5" stroke="currentColor" strokeLinecap="round" strokeLinejoin="round"/>
                    </svg>
                  </span>
                  <label htmlFor="homeowner" className="text-xs mt-[-1px] tracking-wider">Are you a homeowner or resident at Al Mouj</label>
                </div>
              </div>
            </div>

            {data.applicants.map((e, i) => {
              return <React.Fragment key={`applicant-${i + 2}`}>
                <div className="col-span-12">
                  <h3 className="flex items-center text-lg font-semibold mb-4 space-x-2">
                    <span>Applicant {i + 2}</span>
                    <button type="button" onClick={() => removeApplicant(i)} className='text-red-error'>
                      <svg xmlns="http://www.w3.org/2000/svg" class="h-6 w-6" fill="none" viewBox="0 0 24 24" stroke="currentColor">
                        <path strokeLinecap="round" strokeLinejoin="round" strokeWidth="2" d="M19 7l-.867 12.142A2 2 0 0116.138 21H7.862a2 2 0 01-1.995-1.858L5 7m5 4v6m4-6v6m1-10V4a1 1 0 00-1-1h-4a1 1 0 00-1 1v3M4 7h16" />
                      </svg>
                    </button>
                  </h3>
                  <div className="grid grid-cols-12 gap-x-8 gap-y-16 mt-12">
                    <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.applicants[i].first_name} onChange={e => updateApplicant(i, 'first_name', e.target.value)} name="first_name" label="First Name" required={true} error={errors.first_name}/>
                    <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.applicants[i].last_name} onChange={e => updateApplicant(i, 'last_name', e.target.value)} name="last_name" label="Last Name" required={true} error={errors.last_name}/>
                    <Input
                      inputWrapperStyle="col-span-12 md:col-span-6"
                      value={data.applicants[i].email}
                      onChange={e => updateApplicant(i, 'email', e.target.value)}
                      name="email"
                      label="Email"
                      required={true}
                      validate={() => validateEmail(data.applicants[i].email)}
                    />

                    <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.applicants[i].nationality} onChange={e => updateApplicant(i, 'nationality', e.target.value)} name="nationality" label="Nationality" required={true}/>
                    <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.applicants[i].gsm} onChange={e => updateApplicant(i, 'gsm', e.target.value)} name="gsm" label="GSM" required={true}/>
                    <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.applicants[i].phone} onChange={e => updateApplicant(i, 'phone', e.target.value)} name="phone" label="Phone"/>

                    <div className="flex flex-col col-span-12 md:col-span-6 space-y-2">
                      <label className="uppercase text-[10px] sm:text-[13px] tracking-widest">Gender <span className="text-golden">*</span></label>

                      <div className="flex items-center space-x-16">
                        <Checkbox
                          id={`male-${i + 2}`}
                          name="gender"
                          label="Male"
                          checked={data.applicants[i].gender == 'Male'}
                          className="col-span-12 md:col-span-4 flex items-center space-x-2"
                          onChange={e => {
                            updateApplicant(i, 'gender', 'Male')
                          }}
                        />

                        <Checkbox
                          id={`female-${i + 2}`}
                          name="gender"
                          label="Female"
                          checked={data.applicants[i].gender == 'Female'}
                          className="col-span-12 md:col-span-4 flex items-center space-x-2"
                          onChange={e => {
                            updateApplicant(i, 'gender', 'Female')
                          }}
                        />
                      </div>
                    </div>

                    <Input inputWrapperStyle="self-center col-span-12 md:col-span-3 pr-2" value={data.applicants[i].dob} onChange={e => updateApplicant(i, 'dob', e.target.value)} name="dob" label="Date of birth" type="date" required={true}/>

                    <Input inputWrapperStyle="col-span-12 md:col-span-6" value={data.applicants[i].passport} onChange={e => updateApplicant(i, 'passport', e.target.value)} name="passport" label="Resident ID or Passport Number" required={true}/>

                    <div className='col-span-12 md:col-span-6 flex items-center'>
                      <CustomSelect
                        className={"select-small flex-1"}
                        options={handicapOptions()}
                        placeholder={'Handicap'}
                        onChange={(option) => {
                          updateApplicant(i, 'handicap', option.value)
                        }}
                      />
                      <p data-tip="A golf handicap is a numerical measure of a golfer's potential that is used to enable players of varying abilities to compete against one another. Better players are those with the lowest handicaps.">
                      <svg xmlns="http://www.w3.org/2000/svg" className="h-6 w-6 ml-4 text-green-500" fill="none" viewBox="0 0 24 24" stroke="currentColor">
                        <path strokeLinecap="round" strokeLinejoin="round" strokeWidth="2" d="M13 16h-1v-4h-1m1-4h.01M21 12a9 9 0 11-18 0 9 9 0 0118 0z" />
                      </svg>
                      </p>
                      <ReactTooltip className='max-w-[400px]'/>
                    </div>

                    <div className="flex flex-col col-span-12 md:col-span-6 -mt-6">
                      <div className="space-x-2">
                        <input id={`homeowner-${i + 2}`} value={data.applicants[i].homeowner} onChange={e => updateApplicant(i, 'homeowner', e.target.value)} className="border-b p-2" type="checkbox"/>
                        <label htmlFor={`homeowner-${i + 2}`} className="text-sm select-none">Are you are a homeowner or resident at Al Mouj</label>
                      </div>
                    </div>
                  </div>
                </div>
              </React.Fragment>
            })}

            <div className={`col-span-12 md:col-span-6 flex items-center ${isFamily() ? 'justify-between' : 'md:justify-end'}`}>
              {isFamily() && <button type="button" onClick={addApplicant} className="btn btn--outline w-[240px]">Add another applicant</button>}
              <button className="justify-between w-[304px] btn btn--green mt-4">
                Continue to step 2
                <svg xmlns="http://www.w3.org/2000/svg" className="h-6 w-6" fill="none" viewBox="0 0 24 24" stroke="currentColor">
                  <path strokeLinecap="round" strokeLinejoin="round" strokeWidth="2" d="M14 5l7 7m0 0l-7 7m7-7H3" />
                </svg>
              </button>
            </div>

            <p className="col-span-12 md:-mt-12 text-xs text-gray-500"><span className="text-golden">*</span> Required field</p>
          </div>
        </form>
      </div>
    </div>
  )
}

Index.layout = page => <Main width='full' children={page} />

export default Index
