import React, {useEffect} from 'react';
import { useState } from "react";
import Basket from "../../components/basket";
import Alpha from '../../components/typography/Alpha';
import { getBasketTotal } from '../../funcions';
import Main from "../../layouts/Main";
import Sticky from 'react-sticky-el';
import PageHeading from '../../components/PageHeading';
import { InertiaLink, usePage } from '@inertiajs/inertia-react';

const Complete = ({ errors, booking }) => {
  const [isMobile, setIsMobile] = useState({
    matches: window.innerWidth > 768 ? true : false,
  });

  const {prices} = usePage().props;

  useEffect( async () => {
    let mediaQuery = window.matchMedia("(min-width: 1024px)");
    mediaQuery.addEventListener('change', setIsMobile);

    // this is the cleanup function to remove the listener
    return () => mediaQuery.removeEventListener('change', setIsMobile);
  }, [isMobile]);

  return (
    <>
      <PageHeading>
        <Alpha>Complete your booking</Alpha>
        <Sticky
          disabled={!isMobile.matches}
          stickyClassName={"sticky lg:bottom-40"}
          dontUpdateHolderHeightWhenSticky={true}
          className="overflow-y-auto lg:overflow-visible max-h-screen w-full lg:w-auto fixed lg:absolute left:0 lg:left-auto right-0 bottom-0 lg:bottom-auto lg:top-0 z-50"
        >
          <Basket booking={booking} prices={prices} />
        </Sticky>
      </PageHeading>

      <div className='pt-[65px] max-w-[1344px] self-center w-full px-[15px] lg:pr-[575px]'>
        <p>To confirm your booking, please proceed to check out and fill in your card details to submit a payment of <b>OMR {getBasketTotal(booking, prices)} OMR</b>.</p>
        <InertiaLink method="post" as="button" href={`/booking/${booking.id}/complete`} className="mt-6 btn btn--green w-full max-w-[352px] justify-between">
          Proceed to checkout
          <svg xmlns="http://www.w3.org/2000/svg" className="h-6 w-6" fill="none" viewBox="0 0 24 24" stroke="currentColor">
            <path strokeLinecap="round" strokeLinejoin="round" strokeWidth="2" d="M14 5l7 7m0 0l-7 7m7-7H3" />
          </svg>
        </InertiaLink>

        {errors && Object.values(errors).map((error, i) => {
          return <p key={`order-error-${i}`} className="text-red-error mt-4">{error}</p>
        })}
      </div>
    </>
  )
}

Complete.layout = page => <Main className="has-basket" width='full' children={page} />

export default Complete
