import { Inertia } from "@inertiajs/inertia";
import React, { useEffect } from "react";
import { useState } from "react";
import Basket from "../../components/basket";
import Alpha from "../../components/typography/Alpha";
import Main from "../../layouts/Main";
import Sticky from "react-sticky-el";
import { CustomSelect } from "../../components/customSelect";
import { DateTime } from "luxon";
import PageHeading from "../../components/PageHeading";
import { usePage } from "@inertiajs/inertia-react";
import Counter from "../../components/counter";
import { assign, merge } from "lodash";

const Food = ({ food, beverages, golf_manager_product_images }) => {
  const [booking, setBooking] = useState(usePage()?.props?.booking);
  const {prices} = usePage().props;
  const [currentCategory, setCurrentCategory] = useState("food");

  const [isMobile, setIsMobile] = useState({
    matches: window.innerWidth > 768 ? true : false,
  });

  useEffect(async () => {
    let mediaQuery = window.matchMedia("(min-width: 768px)");
    mediaQuery.addEventListener('change', setIsMobile);

    return () => mediaQuery.removeEventListener('change', setIsMobile);
  }, []);

  function clearProducts() {
    setBooking(assign({}, booking, {
      products: assign({}, booking.products, {
        food: {},
        beverages: {},
      }),
    }));
  }

  const timeOptions = () => {
    let time = DateTime.now().set({ hour: 7, minute: 0, seconds: 0 });
    let times = [];

    times.push({id: `option-default`, value: null, label: '-'});

    do {
      time = time.plus({ minutes: 30 });

      times.push({
        id: time.toMillis(),
        value: time.toFormat("HH:mm"),
        label: time.toFormat("h:mma").toLowerCase(),
      });
    } while (time.hour < 17);

    return times;
  };

  const countOptions = () => {
    let counts = [];

    counts.push({id: `option-default`, value: null, label: '-'});

    for (let i = 1; i < 18; i++) {
      counts.push({
        id: `option-${i}`,
        value: i,
        label: i,
      });
    }

    return counts;
  };

  return (
    <>
      <PageHeading>
        <Alpha>Enjoy food & drink during or after your round</Alpha>
        <Sticky
          disabled={!isMobile.matches}
          stickyClassName={"sticky lg:bottom-40"}
          dontUpdateHolderHeightWhenSticky={true}
          className="overflow-y-auto lg:overflow-visible max-h-screen w-full lg:w-auto fixed lg:absolute left:0 lg:left-auto right-0 bottom-0 lg:bottom-auto lg:top-0 z-50"
        >
          <Basket
            booking={booking}
            prices={prices}
            onNext={() => {
              Inertia.post(`/booking/${booking.id}/food`, {
                products: assign({}, booking.products),
              });
            }}
          />
        </Sticky>
      </PageHeading>

      <div className="pt-8 md:pt-16 max-w-[1344px] self-center w-full px-4 lg:pr-[460px] xl:pr-[575px]">
        <section className="mb-8 md:mb-15.6">
          <h3 className="my-4 text-2xl text-green-500">Book your Table?</h3>

          <p className="tracking-tight">
            Would you like to reserve a table after your round?
          </p>

          <div className="flex flex-col w-6/12 md:w-3/12 pt-2.5">
            <label className="tracking-widest uppercase text-[13px]">
              Select a time
            </label>
            <CustomSelect
              defaultValue={booking?.tableReservation?.time}
              options={timeOptions()}
              onChange={(option) => {
                if (option.value) {
                  clearProducts();
                  return;
                }

                setBooking(merge({}, booking, {
                  tableReservation: {
                    time: option.value,
                  },
                }));
              }}
            />
          </div>
        </section>

        <section className="pb-2">
          <h3 className="my-4 text-2xl text-green-500">On Course Deliver</h3>

          <p className="tracking-tight">
            Would you like food and drink delivered to you whilst you're out on
            the course?
          </p>

          <div className="flex flex-col w-6/12 md:w-3/12 pt-2.5">
            <label>Delivery Hole</label>
            <CustomSelect
              className={"select-small"}
              options={countOptions()}
              onChange={(option) => {
                if (!option.value) {
                  clearProducts();
                  return;
                }

                setBooking(merge({}, booking, {
                  foodDelivery: {
                    hole: option.value,
                  },
                }));
              }}
            />
          </div>
        </section>

        {!!booking?.foodDelivery?.hole && (
          <section className="w-full">
            <nav className="flex border-b border-blue-900 border-opacity-40">
              <button
                className={
                  "p-4 pb-3.5 cursor-pointer text-gray-500 min-w-16 text-center mb-[-1px]" +
                  (currentCategory == "food"
                    ? " border-b border-blue-900"
                    : " text-opacity-60")
                }
                onClick={() => setCurrentCategory("food")}
              >
                Food
              </button>
              <button
                className={
                  "p-4 pb-3.5 cursor-pointer text-gray-500 min-w-16 text-center mb-[-1px]" +
                  (currentCategory == "beverages"
                    ? " border-b border-blue-900"
                    : " text-opacity-60")
                }
                onClick={() => setCurrentCategory("beverages")}
              >
                Beverage
              </button>
            </nav>

            <div className="hidden md:flex mt-6">
              <div className="w-16 mr-8"></div>
              <div className="text-xs text-gray-500 flex-grow"></div>
              <div className="text-xs text-gray-500 md:flex-shrink-0 w-29 px-2.5">
                Price
              </div>
              <div className="text-xs text-gray-500 md:flex-shrink-0 w-25 px-2.5">
                Quantity
              </div>
            </div>

            <ul className="divide-y divide-gray-500 divide-opacity-20 divide-dashed pb-10">
              {["food", "beverages"].map((itemType) => (
                <React.Fragment key={itemType}>
                  {currentCategory == itemType &&
                    (itemType == "food" ? food : beverages).map((item) => (
                      <li
                        key={item.id}
                        className="flex w-full py-4 md:pt-6 md:pb-5.5 items-start first:pt-6 md:first:pt-2"
                      >
                        <div className="w-16 h-16 mr-6 md:mr-8 flex-shrink-0">
                          <img
                            className="max-w-none w-full h-full object-cover"
                            src={
                              item.stock_image
                                ? golf_manager_product_images + item.stock_image
                                : "/img/burger.jpg"
                            }
                            width="64"
                            height="64"
                          />
                        </div>

                        <div className="flex flex-col md:flex-row relative md:flex-grow">
                          <div className="flex relative md:flex-grow">
                            <div className="flex flex-col md:pt-2 flex-grow">
                              <h3 className="text-sm mb-2 md:mb-1 pr-25 md:pr-0">
                                {item.name || "Food Item"}
                              </h3>
                              <p className="text-xs text-gray-500">
                                {item.stock_description || 'No description'}
                              </p>
                            </div>

                            <div className="md:flex md:flex-shrink-0 w-29 px-2.5 pr-6 md:pr-2.5 md:pt-5.5 md:self-start absolute right-0 top-0 md:static text-right md:text-left">
                              {(item?.price).toFixed(3)}{" "}
                              <span className="text-xs self-end md:px-1 pb-0.7 leading-normal">
                                OMR
                              </span>
                            </div>
                          </div>

                          <div className="flex justify-between">
                            <div className="flex flex-col items-center md:flex-shrink-0 w-25 pl-0 pr-5 md:pr-3 md:pl-2 pt-3 md:pt-1.5">
                              <Counter
                                value={
                                  booking?.products?.[itemType]?.[item.id]
                                    ?.quantity
                                }
                                onAdd={() => {
                                  setBooking(
                                    merge({}, booking, {
                                      products: {
                                        [itemType]: {
                                          [item.id]: {
                                            name: item.name,
                                            quantity: booking?.products?.[
                                              itemType
                                            ]?.[item.id]?.quantity
                                              ? booking?.products?.[itemType]?.[
                                                  item.id
                                                ]?.quantity + 1
                                              : 1,
                                            price: item.price,
                                            id: item.id,
                                          },
                                        },
                                      },
                                    })
                                  );
                                }}
                                onSubtract={() => {
                                  let tempBooking = merge({}, booking, {
                                    products: {
                                      [itemType]: {
                                        [item.id]: {
                                          name: item.name,
                                          quantity: booking?.products?.[
                                            itemType
                                          ]?.[item.id]?.quantity
                                            ? booking?.products?.[itemType]?.[
                                                item.id
                                              ]?.quantity - 1
                                            : 0,
                                          price: item.price,
                                          id: item.id,
                                        },
                                      },
                                    },
                                  });

                                  Object.values(tempBooking.products[itemType]).forEach((product) => {
                                    if (product.quantity == 0) {
                                      delete tempBooking.products[itemType][product.id]
                                    }
                                  });

                                  setBooking(assign({}, tempBooking));
                                }}
                              />
                            </div>
                          </div>
                        </div>
                      </li>
                    ))}
                </React.Fragment>
              ))}
            </ul>
          </section>
        )}
      </div>
    </>
  );
};

Food.layout = (page) => <Main className="has-basket" width="full" children={page} />;

export default Food;
