import React, {useEffect, useState} from 'react'
import Main from '../../layouts/Main';
import Basket from '../../components/basket';
import Alpha from '../../components/typography/Alpha';
import Charlie from '../../components/typography/Charlie';
import Input from '../../components/input';
import { Inertia } from '@inertiajs/inertia';
import PageHeading from '../../components/PageHeading';
import { CustomSelect, SearchSelect } from '../../components/customSelect';
import PhoneInput from 'react-phone-number-input';
import PhoneTextInput from '../../components/PhoneTextInput';
import 'react-phone-number-input/style.css';
import { usePage } from '@inertiajs/inertia-react';
import { validateEmail } from '../../funcions';
import Sticky from "react-sticky-el";
import { assign } from 'lodash';
import axios from 'axios';
import Counter from '../../components/counter';

const Guest = ({ countries, nationalities }) => {
  const {errors, prices, user, booking} = usePage().props;

  const [loadingPrices, setLoadingPrices] = useState(false);

  const [_booking, setBooking] = useState(booking);

  function handleChange(e, v, w) {
    let value = e.target.value;
    let id = e.target.id;

    if (e.target.type == 'checkbox') {
      value = e.target.checked;
    }

    if (e.type == 'custom-select') {
      id = e.id;
    }

    _booking.guests[v][w][id] = value;

    setBooking(assign({}, _booking));
  }

  function handleSubmit() {
    Inertia.post(`/booking/${booking.id}/guests`, _booking.guests)
  }

  const [isMobile, setIsMobile] = useState({
    matches: window.innerWidth > 768 ? true : false,
  });

  useEffect(async () => {
    let mediaQuery = window.matchMedia("(min-width: 768px)");
    mediaQuery.addEventListener('change', setIsMobile);

    return () => mediaQuery.removeEventListener('change', setIsMobile);
  }, []);

  const addGuest = function (type) {
    _booking.guests[type].push({
      type: 'guest',
      price: prices[type],
    });

    setBooking(assign({}, _booking));
  }

  const removeGuest = function (type) {
    _booking.guests[type].pop();
    setBooking(assign({}, _booking));
  }

  return (
    <>
      <PageHeading>
        <Alpha>Your Guests details</Alpha>
        <Sticky
          disabled={!isMobile.matches}
          stickyClassName={"sticky lg:bottom-40"}
          dontUpdateHolderHeightWhenSticky={true}
          boundaryElement=".footer"
          className="overflow-y-auto lg:overflow-visible max-h-screen w-full lg:w-auto fixed lg:absolute left:0 lg:left-auto right-0 bottom-0 lg:bottom-auto lg:top-0 z-50"
        >
          <Basket booking={_booking} onNext={handleSubmit} disabled={loadingPrices}/>
        </Sticky>
      </PageHeading>

      <div className="pt-8 md:pt-16 max-w-[1344px] self-center w-full px-4 lg:pr-[460px] xl:pr-[500px]">
        <div className='mb-4'>
          <h2 className="mt-4 mb-[9px] lg:mb-[11px]">{user.type == 'Guest' ? `Who's playing?` : `Would you like to add any guests to your tee booking?`}</h2>

          <div className="flex w-full lg:w-auto space-x-[18px] lg:space-x-8">
            <div className="flex flex-col w-[46%]">
              <label className="uppercase text-[10px] sm:text-[13px] text-gray-500 font-bold tracking-widest">Adults</label>
              <div className="flex items-center pr-2 md:pr-0">
                <span className="mr-8 lg:mr-[66px] whitespace-nowrap text-xs">{(prices.adults / 1000).toFixed(3)} OMR</span>

                <Counter
                  value={booking.guests.adults.length}
                  onAdd={() => {
                    addGuest('adults');
                  }}
                  onSubtract={() => {
                    if (_booking.guests.adults.length > 1) {
                      removeGuest('adults')
                    }
                  }}
                  min={1}
                />
              </div>
            </div>

            <div className="flex flex-col w-[46%]">
              <label className="uppercase text-[10px] sm:text-[13px] text-gray-500 font-bold tracking-widest">Juniors</label>
              <div className="flex items-center pr-2 md:pr-0">
                <span className="mr-8 lg:mr-[66px] whitespace-nowrap text-xs">{(prices.juniors / 1000).toFixed(3)} OMR</span>

                <Counter
                  value={booking.guests.juniors.length}
                  onAdd={() => {
                    addGuest('juniors');
                  }}
                  onSubtract={() => {
                    removeGuest('juniors');
                  }}
                />
              </div>
            </div>
          </div>
        </div>

        {Object.keys(_booking.guests).map((guests_type, guests_type_index) =>
          <div key={`${guests_type}-${guests_type_index}`} className="space-y-4">
            {_booking.guests[guests_type].length > 0 && _booking.guests[guests_type].map((guest, guest_index) =>
              <div key={`${guests_type}-${guests_type_index}-${guest_index}`}>
                <div className='flex space-x-4 justify-between items-center'>
                  <Charlie>{guests_type == 'adults' ? 'Adult' : 'Junior'} {guest_index + 1}
                    <small className='text-gray-400 ml-2'>
                      {guest.price != 'loading'
                        ? `${((guest.price !== null ? guest.price : prices[guests_type]) / 1000).toFixed(3)} OMR`
                        : '-'
                      }
                    </small>
                  </Charlie>

                  {!_booking.guests[guests_type][guest_index]?.self && <div className='space-x-4'>
                    {user.type != 'Guest' && <button type='button' className={_booking.guests[guests_type][guest_index]?.type != 'guest' ? 'w-[100px] h-[26px] btn btn--outline px-2' : 'w-[100px] text-gray-400 hover:text-blue-900'} onClick={() => {
                      handleChange({target: {value: 'member', id: 'type'}}, guests_type, guest_index)
                    }}>Member</button>}
                    <button type='button' className={_booking.guests[guests_type][guest_index]?.type == 'guest' ? 'w-[100px] h-[26px] btn btn--outline px-2' : 'w-[100px] text-gray-400 hover:text-blue-900'} onClick={() => {
                      handleChange({target: {value: prices[guests_type], id: 'price'}}, guests_type, guest_index)

                      handleChange({target: {value: 'guest', id: 'type'}}, guests_type, guest_index)
                    }}>Guest</button>
                  </div>}
                </div>

                {(_booking.guests[guests_type][guest_index]?.type && _booking.guests[guests_type][guest_index]?.type == 'member')
                  ? (
                    <div className='grid grid-cols-12 py-4'>
                      {_booking.guests[guests_type][guest_index]?.self
                        ? <div className="col-span-6">{_booking.guests[guests_type][guest_index]?.first_name}</div>
                        : (
                            <SearchSelect
                              className="col-span-6 md:col-span-6 tracking-wider"
                              defaultValue={{
                                label: _booking.guests[guests_type][guest_index]?.first_name ?? 'Member name',
                                id: _booking.guests[guests_type][guest_index]?.id
                              }}
                              onChange={async (option) => {
                                setLoadingPrices(true);
                                handleChange({target: {value: option.id, id: 'id'}}, guests_type, guest_index);

                                handleChange({target: {value: 'loading', id: 'price'}}, guests_type, guest_index);

                                let response = await axios.get(`/api/price?user=${option.id}`);

                                handleChange({target: {value: response.data[guests_type], id: 'price'}}, guests_type, guest_index);

                                setLoadingPrices(false);
                              }}
                              loadOptions={async (search, callback) => {
                                let response = await axios.get(`/api/users?search=${search}`);

                                callback(response.data)
                              }}
                            />
                          )}
                    </div>
                  ) : (
                    <div className="grid grid-cols-12 md:gap-x-12 gap-y-18 mt-8 mb-16">
                      <Input
                        name='first_name'
                        value={_booking.guests[guests_type][guest_index]?.first_name}
                        onChange={e => handleChange(e, guests_type, guest_index)}
                        inputWrapperStyle="col-span-12 md:col-span-6"
                        required={true}
                        label="First Name"
                        error={!!errors[`${guests_type}.${guest_index}.first_name`] && 'First name is required'}
                      />

                      <Input
                        name='last_name'
                        value={_booking.guests[guests_type][guest_index]?.last_name}
                        onChange={e => handleChange(e, guests_type, guest_index)}
                        inputWrapperStyle="col-span-12 md:col-span-6"
                        required={true}
                        label="Last Name"
                        error={!!errors[`${guests_type}.${guest_index}.last_name`] && 'Last name is required'}
                      />

                      <CustomSelect
                        className="col-span-12 md:col-span-6"
                        options={countries}
                        defaultValue={{ label: 'Country of Residence', value: null }}
                        onChange={option => {
                          handleChange({type: 'custom-select', target: option, id: 'country'}, guests_type, guest_index)
                        }}
                      />

                      <CustomSelect
                        className="col-span-12 md:col-span-6 tracking-wider"
                        options={nationalities}
                        defaultValue={{ label: 'Nationality', value: null }}
                        onChange={option => {
                          handleChange({type: 'custom-select', target: option, id: 'nationality'}, guests_type, guest_index)
                        }}
                      />

                      <Input
                        name='email'
                        value={_booking.guests[guests_type][guest_index]?.email}
                        onChange={e => handleChange(e, guests_type, guest_index)}
                        inputWrapperStyle="col-span-12 md:col-span-6"
                        required={true}
                        label="Email Address"
                        error={!!errors[`${guests_type}.${guest_index}.email`] && 'Email address is required'}
                        validate={() => validateEmail(_booking.guests[guests_type][guest_index]?.email)}
                      />

                      <div className="col-span-12 md:col-span-6 flex items-center">
                        <PhoneInput
                          required={true}
                          label={'Phone'}
                          international
                          countryCallingCodeEditable={false}
                          placeholder="Enter phone number"
                          value={_booking.guests[guests_type][guest_index]?.phone}
                          onChange={e => handleChange({target: {value: e, id: 'phone'}}, guests_type, guest_index)}
                          defaultCountry={'OM'}
                          style={{width: '100%', border: 'none'}}
                          inputComponent={PhoneTextInput}
                          error={!!errors[`${guests_type}.${guest_index}.phone`] && 'Phone number is required'}
                        />
                      </div>

                      <div className="col-span-12 space-x-2 flex -mt-8">
                        <div className="custom-checkbox">
                          <input id='marketing' name='marketing' checked={_booking.guests[guests_type][guest_index]?.marketing} onChange={e => handleChange(e, guests_type, guest_index)} type="checkbox" />
                          <span htmlFor="marketing" className="fake-checkbox" onClick={() => {
                            handleChange({
                              target: {
                                value: !guests[guests_type][guest_index]?.marketing,
                                id: 'marketing',
                              }
                            }, guests_type, guest_index)
                          }}>
                            <svg width="12" height="9" viewBox="0 0 12 9" fill="none" xmlns="http://www.w3.org/2000/svg">
                              <path d="M11.0001 1L4.00004 8.00002L1 5" stroke="currentColor" strokeLinecap="round" strokeLinejoin="round"/>
                            </svg>
                          </span>
                          <label className="text-xs mt-[-1px] tracking-wider" htmlFor="marketing">Please tick to opt out of Al Mouj Golf marketing</label>
                        </div>
                      </div>
                    </div>
                  )
                }
              </div>
            )}
          </div>
        )}
      </div>
    </>
  )
}

Guest.layout = page => <Main className="has-basket" width='full' children={page} />

export default Guest
