import React, {useEffect, useState} from 'react'
import { InertiaLink, usePage } from '@inertiajs/inertia-react';
import Main from '../../layouts/Main';
import Basket from '../../components/basket';
import { Inertia } from '@inertiajs/inertia';
import Alpha from '../../components/typography/Alpha';
import Sticky from 'react-sticky-el';
import Counter from '../../components/counter';
import { merge } from 'lodash';

const Hire = ({ booking, club_cost }) => {
  const [_booking, setBooking] = useState(booking);
  const { prices } = usePage()?.props;

  const [isMobile, setIsMobile] = useState({
    matches: window.innerWidth > 768 ? true : false,
  });

  useEffect(async () => {
    let mediaQuery = window.matchMedia("(min-width: 768px)");
    mediaQuery.addEventListener('change', setIsMobile);

    return () => mediaQuery.removeEventListener('change', setIsMobile);
  }, []);

  return (
    <>
      <section className="flex flex-col w-full bg-gray-100">
        <div className="py-[68px] flex flex-col md:flex-row items-center w-full max-w-[1344px] px-[15px] lg:pr-[450px] relative self-center">
          <Alpha>Do you need to hire golf clubs?</Alpha>
          <Sticky
            disabled={!isMobile.matches}
            stickyClassName={"sticky lg:bottom-40"}
            dontUpdateHolderHeightWhenSticky={true}
            className="overflow-y-auto lg:overflow-visible max-h-screen w-full lg:w-auto fixed lg:absolute left:0 lg:left-auto right-0 bottom-0 lg:bottom-auto lg:top-0 z-50"
          >
            <Basket booking={_booking} prices={prices} onNext={() => {
              Inertia.post(`/booking/${booking.id}/hire`, {
                products: {
                  clubs: _booking.products.clubs,
                },
              });
            }}/>
          </Sticky>
        </div>
      </section>

      <div className="pt-[57px] max-w-[1344px] self-center w-full px-[15px] lg:pr-[575px]">
        <label className="uppercase text-[13px] text-gray-500 tracking-widest">Sets of clubs</label>
        <div className="flex items-center">
          <p className="mr-8 md:mr-[67px] whitespace-nowrap text-xs text-gray-500">{ (club_cost).toFixed(3) } OMR</p>
          <Counter
            value={_booking?.products?.clubs}
            onAdd={() => {
              setBooking(merge({}, _booking, {
                products: {
                  clubs: _booking?.products?.clubs ? _booking?.products?.clubs + 1 : 1,
                },
              }));
            }}
            onSubtract={() => {
              setBooking(merge({}, _booking, {
                products: {
                  clubs: _booking?.products?.clubs ? _booking?.products?.clubs - 1 : 0,
                },
              }));
            }}
          />
        </div>

        <div className="flex items-center space-x-2 mt-[25px] mb-[61px]">
          <svg xmlns="http://www.w3.org/2000/svg" className="h-6 w-6 text-golden" fill="none" viewBox="0 0 24 24" stroke="currentColor">
            <path strokeLinecap="round" strokeLinejoin="round" strokeWidth="2" d="M13 16h-1v-4h-1m1-4h.01M21 12a9 9 0 11-18 0 9 9 0 0118 0z" />
          </svg>

          <p className="text-xs text-gray-500">Golf carts are included in the price.</p>
        </div>

        <InertiaLink href={`/booking/${_booking.id}/guests`} className="cursor-pointer flex items-center space-x-2">
          <svg xmlns="http://www.w3.org/2000/svg" className="h-6 w-6 text-green-500" fill="none" viewBox="0 0 24 24" stroke="currentColor">
            <path strokeLinecap="round" strokeLinejoin="round" strokeWidth="2" d="M13 9l3 3m0 0l-3 3m3-3H8m13 0a9 9 0 11-18 0 9 9 0 0118 0z" />
          </svg>

          <p className="text-blue-900 text-[15px]">Skip equipment rental and food & beverage</p>
        </InertiaLink>
      </div>
    </>
  )
}

Hire.layout = page => <Main className="has-basket" width='full' children={page} />

export default Hire
