import { InertiaLink } from '@inertiajs/inertia-react';
import React from 'react';
import Alpha from '../../components/typography/Alpha';
import Main from '../../layouts/Main';

const Success = ({ user, booking }) => {
  return (
    <div className="flex flex-col w-full h-screen">
      <div className="relative h-1/3 w-full flex justify-center">
        <img src="/img/background-course.jpg" className="z-0 absolute w-full h-full object-cover" />
        <div className="max-w-2xl w-full flex justify-end items-end pb-8 relative">
          <Alpha className="text-white">Your booking was successful.</Alpha>
        </div>
      </div>

      <main className="flex justify-center flex-1 w-full py-16">
        <div className="max-w-2xl flex flex-col space-y-4">
          <p>Thanks for booking with Al Mouj Golf, your booking reference number is {booking?.id}, a copy of this confirmation has been sent to bookings@almoujgolf.com.</p>
          <p>We request you to kindly take note of the arrival and cancellation details below.</p>
          <h2 className="font-semibold text-lg">Arrival information</h2>
          <p>Guests are requested to check in at least half an hour before their scheduled tee time.</p>
          <p>Kindly note our COVID-19 measures, please click on the link to view the document:</p>
          <h2 className="font-semibold text-lg">Cancellation information</h2>
          <p>Cancellations or amendments must be done through the reservations link, 72 hours prior to confirmed tee-time.</p>
          <p>If the booking is cancelled or the number of players reduced in less than 72 hours prior to the tee time booked, there will be no refund and 100% charge is applicable.</p>
          <div className="grid grid-cols-5 gap-8">
            <InertiaLink href="/" className="cursor-pointer bg-green-500 py-4 px-8 text-white flex justify-between col-span-3">
                Book another tee time
                <svg xmlns="http://www.w3.org/2000/svg" className="h-6 w-6" fill="none" viewBox="0 0 24 24" stroke="currentColor">
                  <path strokeLinecap="round" strokeLinejoin="round" strokeWidth="2" d="M14 5l7 7m0 0l-7 7m7-7H3" />
                </svg>
            </InertiaLink>
            <a href="https://www.almoujgolf.com/" className="border border-gray-500 col-span-2 flex items-center justify-center">Back to homepage</a>
          </div>
        </div>
      </main>
    </div>
  )
}

Success.layout = page => <Main width='full' children={page} />

export default Success
