import React, { useEffect, useState } from 'react';
import { InertiaLink, useForm } from '@inertiajs/inertia-react';
import Main from '../layouts/Main';
import Alpha from '../components/typography/Alpha';
import { Inertia } from '@inertiajs/inertia';
import Input from '../components/input';

const Login = () => {
  const { data, setData, post, processing, errors } = useForm({
    email: '',
    password: '',
  });

  const [isMobile, setIsMobile] = useState({
    matches: window.innerWidth >= 1024 ? true : false,
  });

  useEffect( async () => {
    let mediaQuery = window.matchMedia("(min-width: 1024px)");
    mediaQuery.addEventListener('change', setIsMobile);
    // this is the cleanup function to remove the listener
    return () => mediaQuery.removeEventListener('change', setIsMobile);
  }, [isMobile]);

  const [activeTab, setActiveTab] = useState('member');

  async function submit(e) {
    e.preventDefault()
    post('/login')
  }

  return (
      <section className="bg-gray-100 h-full p-8 flex justify-center items-center">
        <div className="flex flex-col md:flex-row w-full md:w-[1088px] h-[572px]">
          <ul className="flex md:hidden w-full border-b">
            <li className={`py-3 px-6 border-b mb-[-1px] ${activeTab == 'member' && 'border-blue-900'}`} onClick={() => setActiveTab('member')}>Members</li>
            <li className={`py-3 px-6 border-b mb-[-1px] ${activeTab == 'guest' && 'border-blue-900'}`} onClick={() => setActiveTab('guest')}>Guests</li>
          </ul>

          {(isMobile?.matches || (!isMobile.matches && activeTab == 'member')) && <div className="bg-white flex-1 p-8 md:p-16">
            <Alpha>Member sign in</Alpha>

            <form onSubmit={submit} className="flex flex-col mt-12">
              <Input
                value={data.email}
                onChange={e => setData('email', e.target.value)}
                className="border-b border-gray-300 py-2 w-4/5 outline-none uppercase"
                label="Email Address *"
              />
              {errors && errors.email && <p className="text-red-error mt-2">{ errors.email }</p>}

              <div className="mt-12">
                <Input
                  value={data.password} onChange={e => setData('password', e.target.value)}
                  className="border-b border-gray-300 py-2 w-4/5 outline-none uppercase"
                  type="password"
                  label="Password *"
                />
                {errors && errors.password && <p className="text-red-error mt-2">{ errors.password }</p>}
                {errors && errors.login && <p className="text-red-error mt-2">{ errors.login }</p>}
              </div>

              <InertiaLink className="mt-8 underline text-gray-400 text-sm" href="/password">Forgotten your password?</InertiaLink>

              <p className="mt-4 text-gray-400 text-sm">* Required Field</p>

              <button disabled={processing} className={`mt-16 btn w-[240px] justify-center ${processing ? 'cursor-wait border border-gray-200 text-gray-200' : 'btn--outline'}`}>
                <span>Log In</span>
              </button>
            </form>
          </div>}

          {(isMobile?.matches || (!isMobile?.matches && activeTab == 'guest')) && <div className="p-8 md:p-16 text-white bg-blue-900 w-full md:w-[448px] ">
            <Alpha>Play as a guest</Alpha>

            <p className="text-sm mt-4 md:mt-8">Not a member at our club? Log in here to play as a guest</p>

            <button type='button' onClick={() => {
              Inertia.post('/login/guest');
            }} className="btn btn--green w-[240px] mt-10">Continue</button>
          </div>}
        </div>
      </section>
  )
}

Login.layout = page => <Main width='full' children={page} />

export default Login
