const REM_BASE = 16
const round = num =>
  num
    .toFixed(7)
    .replace(/(\.[0-9]+?)0+$/, '$1')
    .replace(/\.0$/, '')
const rem = (px, base = REM_BASE) => `${round(px / base)}rem`

module.exports = {
  mode: 'jit',
  content: [
    './pages/**/*.{js,ts,jsx,tsx}',
    './components/**/*.{js,ts,jsx,tsx}',
    './resources/**/*.blade.php',
    './resources/**/*.jsx',
  ],
  safelist: ['[class*="react-select"]'],
  media: false, // or 'media' or 'class'
  theme: {
    extend: {
      fontSize: {
        'xs': ['0.75rem', { lineHeight: '1.125rem' }],
        "sm": ['0.9375rem', { lineHeight: '1.625rem' }],
        'base': ['1rem', { lineHeight: '1.625rem' }],
        '2xxl': rem(28),
        '4xxl': rem(40),
      },
      zIndex: {
        '-1': '-1',
        1: '1'
      },
      scale: {
        '-100': '-1',
      },
      minHeight: {
        '4': '2rem',
        '4.5': '2.25rem',
      },
      minWidth: {
        '16': '8rem',
      },
      spacing: {
        0.7: '0.1875rem',
        4.5: '1.125rem',
        5.5: '1.375rem',
        6.5: '1.625rem',
        11.5: '2.875rem',
        15.5: '3.875rem',
        15.6: '3.9375rem',
        18: '4.375rem',
        25: '6.25rem',
        29: '7.25rem',
      },
      lineHeight: {
        30: rem(30),
      }
    },
    screens: {
      sm: '375px',
      md: '768px',
      lg: '1024px',
      xl: '1342px',
    },
    colors: {
      transparent: 'transparent',
      white: '#FFF',
      golden: '#B4975A',
      gray: {
        100: '#F3F3F5',
        200: '#D5D5D9',
        400: '#747480',
        500: '#5C5C69',
      },
      green: {
        500: '#00A976',
      },
      blue: {
        900: '#14133B',
      },
      red: {
        error: '#B66668'
      }
    },
    fontFamily: {
      sans: [
        'Nexa',
        '-apple-system',
        'BlinkMacSystemFont',
        '"Segoe UI"',
        'Roboto',
        '"Helvetica Neue"',
        'Arial',
        '"Noto Sans"',
        'sans-serif',
        '"Apple Color Emoji"',
        '"Segoe UI Emoji"',
        '"Segoe UI Symbol"',
        '"Noto Color Emoji"',
      ],
      serif: ['Canela', 'Georgia', 'Cambria', '"Times New Roman"', 'Times', 'serif'],
    },
    letterSpacing: {
      tight: '-0.01em',
      wide: '0.011em',
      wider: '0.02em',
      widest: '0.2em',
    },
  },
  variants: {
    extend: {},
  },
  corePlugins: {},
  plugins: [
    require('@tailwindcss/forms'),
  ],
}
