<?php

namespace Tests\Unit;

use App\Models\User;
use Carbon\Carbon;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Support\Facades\Auth;
use Tests\TestCase;
use Nette\Utils\Random;

class BookingControllerTest extends TestCase
{
    use DatabaseMigrations;

    protected function setUp(): void
    {
        parent::setUp();

        $user = User::factory()->create();
        Auth::login($user);
    }

    /**
     * @test
     */
    public function can_create_booking_session()
    {
        $id = Random::generate(10);

        $response = $this->post('/booking', [
            'id' => $id,
            'start' => Carbon::now()->addDay(1)->toIso8601String(),
        ]);

        $response->assertStatus(302);
        $response->assertRedirect(route('booking.guests.index', [
            'booking' => $id,
        ]));
    }

    /**
     * @test
     */
    public function cant_create_booking_session_in_the_past()
    {
        $id = Random::generate(10);

        $response = $this->post('/booking', [
            'id' => $id,
            'start' => Carbon::now()->subDay(1)->toIso8601String(),
        ]);

        $response->assertStatus(302);
        $response->assertSessionHasErrors();
        $response->assertRedirect(route('availability.index'));
    }
}
