<?php

namespace Tests\Unit;

use App\Models\User;
use Carbon\Carbon;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Support\Facades\Auth;
use Inertia\Testing\Assert;
use Tests\TestCase;
use Nette\Utils\Random;

class HireControllerTest extends TestCase
{
    use DatabaseMigrations;

    public $id;

    protected function setUp(): void
    {
        parent::setUp();

        $this->id = Random::generate(10);

        $user = User::factory()->create();
        Auth::login($user);

        $this->post('/booking', [
            'id' => $this->id,
            'guests' => [
                'adults' => [],
                'juniors' => [],
            ],
            'start' => Carbon::now()->addDays(1),
        ]);
    }

    public function can_see_hire_page()
    {
        $response = $this->get(route('booking.hire.index', [
            'booking' => $this->id,
        ]));

        $response->assertStatus(200);
        $response->assertInertia(fn (Assert $page) => $page
            ->component('Booking/Hire'));
    }

    /**
     * Add clubs or equipment to a booking session
     *
     * @test
     */
    public function can_add_hire_equipment_to_booking_session()
    {
        $response = $this->post(route('booking.hire.store', [
            'booking' => $this->id,
        ]), [
            'products' => [
                'clubs' => 1,
            ],
        ]);

        $response->assertStatus(302);
        $response->assertRedirect(route('booking.food.index', [
            'booking' => $this->id,
        ]));
    }
}
