<?php

namespace Tests\Unit;

use App\Models\User;
use Carbon\Carbon;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Support\Facades\Auth;
use Inertia\Testing\Assert;
use Tests\TestCase;
use Nette\Utils\Random;

class LoginControllerTest extends TestCase
{
    use DatabaseMigrations;

    /**
     * User is redirected if they are already logged in
     *
     * @test
     */
    public function is_redirected_if_logged_in()
    {
        $user = User::factory()->create();
        Auth::login($user);

        $response = $this->get('/login');

        $response->assertStatus(302);
        $response->assertRedirect(route('availability.index'));
    }

    /**
     * Can view the Inertiajs login page
     *
     * @test
     */
    public function can_see_login_page_if_not_logged_in()
    {
        $response = $this->get('/login');

        $response->assertStatus(200);
        $response->assertInertia(fn (Assert $page) => $page
            ->component('Login'));
    }
}
